@@ -31,6 +31,7 @@
 #include <assert.h>
 #include <string.h>
 #include <stdlib.h>
+#include <mutex>
 
 
 // TODO: should be in some vps.c related header
@@ -168,12 +169,22 @@ LIBDE265_API int de265_isOK(de265_error err)
 
 
 
-ALIGNED_8(static de265_sync_int de265_init_count) = 0;
+static int de265_init_count;
+
+static std::mutex& de265_init_mutex()
+{
+  static std::mutex de265_init_mutex;
+  return de265_init_mutex;
+}
+
 
 LIBDE265_API de265_error de265_init()
 {
-  int cnt = de265_sync_add_and_fetch(&de265_init_count,1);
-  if (cnt>1) {
+  std::lock_guard<std::mutex> lock(de265_init_mutex());
+
+  de265_init_count++;
+
+  if (de265_init_count > 1) {
     // we are not the first -> already initialized
 
     return DE265_OK;
@@ -185,7 +196,7 @@ LIBDE265_API de265_error de265_init()
   init_scan_orders();
 
   if (!alloc_and_init_significant_coeff_ctxIdx_lookupTable()) {
-    de265_sync_sub_and_fetch(&de265_init_count,1);
+    de265_init_count--;
     return DE265_ERROR_LIBRARY_INITIALIZATION_FAILED;
   }
 
@@ -194,13 +205,15 @@ LIBDE265_API de265_error de265_init()
 
 LIBDE265_API de265_error de265_free()
 {
-  int cnt = de265_sync_sub_and_fetch(&de265_init_count,1);
-  if (cnt<0) {
-    de265_sync_add_and_fetch(&de265_init_count,1);
+  std::lock_guard<std::mutex> lock(de265_init_mutex());
+
+  if (de265_init_count<=0) {
     return DE265_ERROR_LIBRARY_NOT_INITIALIZED;
   }
 
-  if (cnt==0) {
+  de265_init_count--;
+
+  if (de265_init_count==0) {
     free_significant_coeff_ctxIdx_lookupTable();
   }
 
@@ -620,10 +633,10 @@ LIBDE265_API int de265_get_bits_per_pixel(const struct de265_image* img,int chan
 {
   switch (channel) {
   case 0:
-    return img->sps.BitDepth_Y;
+    return img->get_sps().BitDepth_Y;
   case 1:
   case 2:
-    return img->sps.BitDepth_C;
+    return img->get_sps().BitDepth_C;
   default:
     return 0;
   }
